import * as d3 from "d3";
import React from "react";

/**
 * Fonction dessinant l'axe des abscisses
 * @param param0 
 * @returns 
 */
export function AxisX({ticks, transform}: { ticks:{ value: Date, xOffset: number }[], transform: string}) {
    let format = {day:"numeric", month: "numeric", year: "numeric"};
    if(ticks.length > 1){
        //Si les 2 premiers ticks ne font pas parties du même mois mais de la même année, on affiche le mois
        if(ticks[0].value.getMonth() !== ticks[1].value.getMonth() && ticks[0].value.getFullYear() === ticks[1].value.getFullYear())
            format = {month: "long", year: "numeric"};
        //Sinon, on affiche uniquement l'année
        else if (ticks[0].value.getFullYear() !== ticks[1].value.getFullYear())
            format = {year: "numeric"};
    }
    console.log(format);
    return (
        <g
            key="xAxis"
            transform={transform}
        >
            {ticks.map(({ value, xOffset }) => (
                <g
                    key={value.toISOString()}
                    transform={`translate(${xOffset}, 0)`}
                >
                    <line
                        y2="6"
                        stroke="currentColor"
                    />
                    <text
                        key={value.toISOString()}
                        style={{
                            fontSize: "10px",
                            textAnchor: "middle",
                            transform: "translateY(20px)"
                        }}>
                        {new Intl.DateTimeFormat('fr-FR', format).format(value)}
                    </text>
                </g>
            ))}
            {(ticks.length>0)?(
                <g>
                    <line 
                        x1="0"
                        x2={ticks[ticks.length-1].xOffset}
                        stroke="currentColor"
                    />
                </g>
            ):<g />}
            
        </g>
    )
}

/**
 * Fonction dessinant l'axe des ordonnées
 * @param param0 
 * @returns 
 */
export function AxisY({ticks, transform, textLabel}: { ticks:{ value: any, yOffset: number }[], transform: string, textLabel:any}){
    return (
        <g
            key="yAxis"
            transform={transform}
        >
            {(ticks.length>0)?(
                <text
                    style={{
                        fontSize: "26px",
                        textAnchor: "middle",
                        transform: `translate(-85px,${ticks[ticks.length-1].yOffset/2-55}px)`
                    }}
                >
                    {textLabel}
                </text>
            ):<text />}
            {ticks.map(({ value, yOffset }) => (
                <g
                    key={value}
                    transform={`translate(-6, ${yOffset})`}
                >
                    <line
                        x2="6"
                        stroke="currentColor"
                    />
                    <text
                        key={value}
                        style={{
                            fontSize: "10px",
                            textAnchor: "middle",
                            transform: "translateX(-10px)"
                        }}>
                        {value}
                    </text>
                </g>
            ))}
            {(ticks.length>0)?(
                <g>
                    <line 
                        y1="0"
                        y2={ticks[ticks.length-1].yOffset}
                        stroke="currentColor"
                    />
                </g>
            ):<g />}
        </g>
    )
}